/*@@@@@@@@@@@@@@@@@@@@@@ EuropeanStrike.h @@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 Random Option Pricing
 Summer 2002
 Authors: A. Korobchevsky
          S. Ferrando
 Ryerson University ALL RIGHTS RESERVED
 
 Purpose: Implements the European Average Strike option 

 Instance variables:
	  sum_St - total stock price for all timesteps
	       N - number of timesteps
 @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

#ifndef ER_STRIKE_H__
#define ER_STRIKE_H__

#include "Derivative.h"

struct AverageStrikeEuropeanCall : Derivative
{ 
  


  /*********************** EuropeanStrike *****************************
   * Function: EuropeanStrike
   * Purpose: Constructor
   * Param: Total number of timesteps
   * Returns: N/A
   ********************************************************************/
  AverageStrikeEuropeanCall(int N)
  { this->N = N;
    sum_St = 0;
  }




  /*********************** init ***************************************
   * Function: init
   * Purpose: Initialises sum_St utility parameter
   * Param: N/A
   * Returns: N/A
   * Note: Empty for path independent derivatives
   ********************************************************************/
  void init()
  { sum_St = 0;
  }




  /********************* payOff ****************************************
   * Function: payOff
   * Purpose: Returns the option payoff
   * Param: Final stock price
   * Returns: option payoff
   * Precondiion: Underlying and derivative parameter must be initialized
   * Note: Call only after all timesteps are completed
   ********************************************************************/
  double payOff(double St)
  { //cout << "sum_St: " << sum_St << ", St: " << St << ", Difference: " << St - sum_St / (N+1) << endl;
    return St - sum_St / (N+1) > 0 ? St - sum_St / (N+1) : 0;
  }




  /*********************** update *************************************
   * Function: update
   * Purpose: Updates the sum values between timesteps
   * Param: Stock value at any given point
   * Returns: N/A
   * Note: Empty for path independent derivatives
   ********************************************************************/
  void update(double St)
  { sum_St += St;
  //cout << "Update (regular version): sum_St = " << sum_St << endl;
  }


  private:
    int N;
};

#endif
